@@ -3,11 +3,14 @@
 #' @description Perform leave-one-out cross validation with options for computationally
 #'   efficient approximations for big data.
 #'
-#' @param object A fitted model object from [splm()] or [spautor()].
+#' @param object A fitted model object from [splm()], [spautor()], [spglm()], or [spgautor()].
 #' @param cv_predict A logical indicating whether the leave-one-out fitted values
-#'   should be returned. Defaults to \code{FALSE}.
+#'   should be returned. Defaults to \code{FALSE}. If \code{object} is from [spglm()] or [spgautor()],
+#'   the fitted values returned are on the link scale.
 #' @param se.fit A logical indicating whether the leave-one-out
 #'   prediction standard errors should be returned. Defaults to \code{FALSE}.
+#'   If \code{object} is from [spglm()] or [spgautor()],
+#'   the standard errors correspond to the fitted values returned on the link scale.
 #' @param local A list or logical. If a list, specific list elements described
 #'   in [predict.spmodel()] control the big data approximation behavior.
 #'   If a logical, \code{TRUE} chooses default list elements for the list version
@@ -30,6 +33,7 @@
 #'   and \code{se.fit}, a numeric vector with leave-one-out prediction standard
 #'   errors for each observation (if \code{se.fit = TRUE}).
 #'
+#' @order 1
 #' @export
 #'
 #' @examples
@@ -45,9 +49,9 @@ loocv <- function(object, ...) {
 
 #' @rdname loocv
 #' @method loocv splm
+#' @order 2
 #' @export
 loocv.splm <- function(object, cv_predict = FALSE, se.fit = FALSE, local, ...) {
-
   if (missing(local)) {
     local <- NULL
   }
@@ -66,21 +70,22 @@ loocv.splm <- function(object, cv_predict = FALSE, se.fit = FALSE, local, ...) {
   local_list <- get_local_list_prediction(local)
 
   if (local_list$method == "all") {
-    spcov_params_val <- coef(object, type = "spcov")
-    dist_matrix <- spdist(object$obdata, object$xcoord, object$ycoord)
-    randcov_params_val <- coef(object, type = "randcov")
-    if (is.null(object$random)) {
-      randcov_names <- NULL
-      randcov_Zs <- NULL
-    } else {
-      randcov_names <- get_randcov_names(object$random)
-      randcov_Zs <- get_randcov_Zs(object$obdata, randcov_names)
-    }
-    partition_matrix_val <- partition_matrix(object$partition_factor, object$obdata)
-    cov_matrix_val <- cov_matrix(
-      spcov_params_val, dist_matrix, randcov_params_val,
-      randcov_Zs, partition_matrix_val
-    )
+    # spcov_params_val <- coef(object, type = "spcov")
+    # dist_matrix <- spdist(object$obdata, object$xcoord, object$ycoord)
+    # randcov_params_val <- coef(object, type = "randcov")
+    # if (is.null(object$random)) {
+    #   randcov_names <- NULL
+    #   randcov_Zs <- NULL
+    # } else {
+    #   randcov_names <- get_randcov_names(object$random)
+    #   randcov_Zs <- get_randcov_Zs(object$obdata, randcov_names)
+    # }
+    # partition_matrix_val <- partition_matrix(object$partition_factor, object$obdata)
+    # cov_matrix_val <- cov_matrix(
+    #   spcov_params_val, dist_matrix, randcov_params_val,
+    #   randcov_Zs, partition_matrix_val
+    # )
+    cov_matrix_val <- covmatrix(object)
 
     # actually need inverse because of HW blocking
     cov_matrixInv_val <- chol2inv(chol(forceSymmetric(cov_matrix_val)))
@@ -94,16 +99,16 @@ loocv.splm <- function(object, cv_predict = FALSE, se.fit = FALSE, local, ...) {
     if (local_list$parallel) {
       cl <- parallel::makeCluster(local_list$ncores)
       cv_predict_val_list <- parallel::parLapply(cl, seq_len(object$n), get_loocv,
-                                                 Sig = cov_matrix_val,
-                                                 SigInv = cov_matrixInv_val, Xmat = X, y = y, yX = yX,
-                                                 SigInv_yX = SigInv_yX, se.fit = se.fit
+        Sig = cov_matrix_val,
+        SigInv = cov_matrixInv_val, Xmat = X, y = y, yX = yX,
+        SigInv_yX = SigInv_yX, se.fit = se.fit
       )
       cl <- parallel::stopCluster(cl)
     } else {
       cv_predict_val_list <- lapply(seq_len(object$n), get_loocv,
-                                    Sig = cov_matrix_val,
-                                    SigInv = cov_matrixInv_val, Xmat = X, y = y, yX = yX,
-                                    SigInv_yX = SigInv_yX, se.fit = se.fit
+        Sig = cov_matrix_val,
+        SigInv = cov_matrixInv_val, Xmat = X, y = y, yX = yX,
+        SigInv_yX = SigInv_yX, se.fit = se.fit
       )
     }
     # cv_predict_val <- unlist(cv_predict_val_list)
@@ -147,10 +152,11 @@ loocv.splm <- function(object, cv_predict = FALSE, se.fit = FALSE, local, ...) {
   cv_output
 }
 
+#' @rdname loocv
 #' @method loocv spautor
+#' @order 3
 #' @export
 loocv.spautor <- function(object, cv_predict = FALSE, se.fit = FALSE, local, ...) {
-
   if (missing(local)) {
     local <- NULL
   }
@@ -158,22 +164,23 @@ loocv.spautor <- function(object, cv_predict = FALSE, se.fit = FALSE, local, ...
   local_list <- get_local_list_prediction(local)
 
   # local not used but needed for S3
-  spcov_params_val <- coef(object, type = "spcov")
-  dist_matrix <- object$W
-  randcov_params_val <- coef(object, type = "randcov")
-  if (is.null(object$random)) {
-    randcov_names <- NULL
-    randcov_Zs <- NULL
-  } else {
-    randcov_names <- get_randcov_names(object$random)
-    randcov_Zs <- get_randcov_Zs(object$data, randcov_names)
-  }
-  partition_matrix_val <- partition_matrix(object$partition_factor, object$data)
-  cov_matrix_val <- cov_matrix(
-    spcov_params_val, dist_matrix, randcov_params_val,
-    randcov_Zs, partition_matrix_val, object$M
-  )
-  cov_matrix_obs_val <- cov_matrix_val[object$observed_index, object$observed_index, drop = FALSE]
+  # spcov_params_val <- coef(object, type = "spcov")
+  # dist_matrix <- object$W
+  # randcov_params_val <- coef(object, type = "randcov")
+  # if (is.null(object$random)) {
+  #   randcov_names <- NULL
+  #   randcov_Zs <- NULL
+  # } else {
+  #   randcov_names <- get_randcov_names(object$random)
+  #   randcov_Zs <- get_randcov_Zs(object$data, randcov_names)
+  # }
+  # partition_matrix_val <- partition_matrix(object$partition_factor, object$data)
+  # cov_matrix_val <- cov_matrix(
+  #   spcov_params_val, dist_matrix, randcov_params_val,
+  #   randcov_Zs, partition_matrix_val, object$M
+  # )
+  # cov_matrix_obs_val <- cov_matrix_val[object$observed_index, object$observed_index, drop = FALSE]
+  cov_matrix_obs_val <- covmatrix(object)
 
   # actually need inverse because of HW blocking
   cov_matrixInv_obs_val <- chol2inv(chol(forceSymmetric(cov_matrix_obs_val)))
@@ -186,16 +193,16 @@ loocv.spautor <- function(object, cv_predict = FALSE, se.fit = FALSE, local, ...
   if (local_list$parallel) {
     cl <- parallel::makeCluster(local_list$ncores)
     cv_predict_val_list <- parallel::parLapply(cl, seq_len(object$n), get_loocv,
-                                               Sig = cov_matrix_obs_val,
-                                               SigInv = cov_matrixInv_obs_val, Xmat = X, y = y, yX = yX,
-                                               SigInv_yX = SigInv_yX, se.fit = se.fit
+      Sig = cov_matrix_obs_val,
+      SigInv = cov_matrixInv_obs_val, Xmat = X, y = y, yX = yX,
+      SigInv_yX = SigInv_yX, se.fit = se.fit
     )
     cl <- parallel::stopCluster(cl)
   } else {
     cv_predict_val_list <- lapply(seq_len(object$n), get_loocv,
-                                  Sig = cov_matrix_obs_val,
-                                  SigInv = cov_matrixInv_obs_val, Xmat = X, y = y, yX = yX,
-                                  SigInv_yX = SigInv_yX, se.fit = se.fit
+      Sig = cov_matrix_obs_val,
+      SigInv = cov_matrixInv_obs_val, Xmat = X, y = y, yX = yX,
+      SigInv_yX = SigInv_yX, se.fit = se.fit
     )
   }
   # cv_predict_val <- unlist(cv_predict_val_list)
@@ -217,7 +224,6 @@ loocv.spautor <- function(object, cv_predict = FALSE, se.fit = FALSE, local, ...
     }
   }
   cv_output
-
 }
 
 loocv_local <- function(row, object, se.fit, local_list) {
